//@author: vux
//@help: template for standard shaders
//@tags: template
//@credits: 

Texture2D texture2d <string uiname="Texture";>;

SamplerState linearSampler : IMMUTABLE
{
    Filter = MIN_MAG_MIP_LINEAR;
    AddressU = Clamp;
    AddressV = Clamp;
};
 
cbuffer cbPerDraw : register( b0 )
{
	float4x4 tVP : VIEWPROJECTION;
	float threshold = 0.5;
	float fading = 0.5;
};

cbuffer cbPerObj : register( b1 )
{
	float4x4 tW : WORLD;
	float4 cAmb <bool color=true;String uiname="Color";> = { 1.0f,1.0f,1.0f,1.0f };
};

struct VS_IN
{
	float4 PosO : POSITION;
	float4 TexCd : TEXCOORD0;
	float4 Color : COLOR;

};

struct vs2ps
{
    float4 PosWVP: SV_POSITION;
    float4 TexCd: TEXCOORD0;
	float4 Color : COLOR;
};

vs2ps VS(VS_IN input)
{
    vs2ps output;
    output.PosWVP  = mul(input.PosO,mul(tW,tVP));
    output.TexCd = input.TexCd;
    output.Color = input.Color;
    return output;
}

float4 PS_threshIn(vs2ps In): SV_Target
{
	float4 col = float4(0,0,0,0);
	if (In.Color.r < threshold) {
		discard;
	} else {
		col = texture2d.Sample(linearSampler,In.TexCd.xy) * cAmb;
	}
	return col;
}

float scale(float val,float x, float y, float a, float b) {
	return (b-a)*(val-x)/(y-x)+a;
}


float4 PS_fadeIn(vs2ps In): SV_Target
{
	float4 col = float4(0,0,0,0);
	if (In.Color.r < threshold-fading*0.1) {
		discard;
	} else if (In.Color.r < threshold && In.Color.r > fading*0.1) {
		float fade = scale(In.Color.r, threshold-fading*0.1, threshold, 0, 1);
		col = texture2d.Sample(linearSampler,In.TexCd.xy) * cAmb;
		col.a = fade;
	} else {
		col = texture2d.Sample(linearSampler,In.TexCd.xy) * cAmb;
	}
	return col;
}

float4 PS_color(vs2ps In): SV_Target
{
    float4 col = In.Color;
    return col;
}





technique10 fadeIn
{
	pass P0
	{
		SetVertexShader( CompileShader( vs_4_0, VS() ) );
		SetPixelShader( CompileShader( ps_4_0, PS_fadeIn() ) );
	}
}

technique10 threshIn
{
	pass P0
	{
		SetVertexShader( CompileShader( vs_4_0, VS() ) );
		SetPixelShader( CompileShader( ps_4_0, PS_threshIn() ) );
	}
}


technique10 color
{
	pass P0
	{
		SetVertexShader( CompileShader( vs_4_0, VS() ) );
		SetPixelShader( CompileShader( ps_4_0, PS_color() ) );
	}
}




